/* ------------------------------------------------- */
/*                    CLEARHD.C                      */
/*  CLEARHD berprft, ob sich die Dateien, die in   */
/*  der angegebenen Schlsseldatei vorhanden sind,   */
/*  in Gre, Datum oder Uhrzeit gendert haben, ob  */
/*  sie berhaupt noch existieren und ob es Dateien  */
/*  gibt, die sich nicht in der Schlsseldatei be-   */
/*  finden. Bei nicht in der Schlsseldatei vorhan-  */
/*  denen Dateien wird das Lschen der entsprechen-  */
/*  den Dateien angeboten, wenn der entsprechende    */
/*  Schalter gesetzt wird.                           */
/* ------------------------------------------------- */
/*  Parameter                                        */
/*  Eingabe   : argc    : Anzahl der bergebenen     */
/*                        Parameter                  */
/*              *argv[] : Zeiger auf die bergebenen */
/*                        Parameter                  */
/*   Ausgabe   : Bei fehlerhaftem Aufruf wird das    */
/*               Programm abgebrochen und            */
/*               setzt mit exit() den ERRORLEVEL     */
/*               auf 1                               */
/* ------------------------------------------------- */
/*        (c) 1991 Dirk Alpers & DMV-Verlag          */
/* ------------------------------------------------- */

#include "def.h"   /* enthlt alle ntigen Prototypen
                      und Konstanten */

/* --- Typen --------------------------------------- */
struct pointer {
         char    name[80];
  struct pointer *next;
};

/* --- Prototypen ---------------------------------- */
void           Parameter_auswerten(int argc,
                                   char *argv[],
                                   char *source);
void           Anleitung_ausgeben(void);
struct pointer *vergleichen(struct pointer *rootptr,
                            char *dir);
int            cutstr(char *dest, char *source,
                      int position, int length);

/* --- Globale Variablen --------------------------- */
int    OK      = FALSE,
          /* Bei korrekten Files OK ausgeben?        */
       DELETE  = FALSE,
          /* Bei unkorrekten Files lschen anbieten? */
       NOQUERY = FALSE;
          /* Lschen ohne Sicherheitsabfrage?        */

/* ------------------------------------------------- */
void main(int argc,char *argv[])
{
  char   file[MAXPATH],
           /* Zeile aus der Schlsseldatei           */
         dir[MAXPATH];
           /* zu durchsuchendes Directory            */
  char   source[MAXPATH];
           /* Name der Schlsseldatei                */
  FILE   *fp;
           /* Filepointer der Schlsseldatei         */
  struct pointer *rootptr,
           /* Anfang der einfach verketteten Liste   */
                 *helpptr,
           /* Hilfszeiger zum Aufbau und Durchlaufen */
                 *laufzeiger;
           /*   der einfach verketteten Liste        */

  printf("\nCLEARHD 1.0 (c) 1991 Alpers/Dietl");
  Parameter_auswerten(argc, argv, source);

  fp = fopen(source, "r");
                  /* Schlsseldatei zum Lesen ffnen */
  if (errno == 0) {
           /* Konnte Schlsseldatei geffnet werden? */
    fscanf(fp, "%s", file);
          /* Ersten Eintrag aus Schlsseldatei lesen */
    while (!feof(fp)) {
      strcpy(dir, file+1);
                         /* "*" aus String streichen */
      printf("\nDir: %s", dir);
        /* Zu durchsuchenden Directorynamen ausgeben */
      rootptr = NULL;      /* rootptr initialisieren */
      laufzeiger = NULL;
      fscanf(fp, "%s", file);
      /* Ersten Fileeintrag aus Schlsseldatei lesen */
      /* Alle Fileeintrge aus einem Directory in    */
      /* einer Pointerkette ablegen                  */
      while((file[0] != '*') && (!feof(fp)))
      {
        helpptr = (struct pointer *)
                  malloc(sizeof(struct pointer));
        strcpy(helpptr->name,file);
        helpptr->next = NULL;
        if (rootptr == NULL)
          rootptr = helpptr;
        else
          laufzeiger->next = helpptr;
        laufzeiger = helpptr;
        fscanf(fp,"%s",file);
        /* Nchsten Eintrag aus Schlsseldatei holen */
      }
      errno = 0;

      rootptr = vergleichen(rootptr, dir);

      /* Nicht gefundene Dateien ausgeben und        */
      /* Pointer-Speicherplatz freigeben             */
      helpptr = rootptr;
      while (helpptr)
      {
        printf("\n%s  *** MISSING ***",helpptr->name);

        laufzeiger = helpptr->next;
        free(helpptr);
                  /* Pointer-Speicherplatz freigeben */
        helpptr = laufzeiger;
      }
    }

    fclose(fp);          /* Schlsseldatei schlieen */
  }
  else
    printf("\nCan't open Keyfile\n");
      /* Schlsseldatei konnte nicht geffnet werden */
  printf("\n");
} /* clearhd */


/* ------------------------------------------------- */
/*  Modulname : Parameter_auswerten                  */
/*  Aufgabe   : Auswerten der dem Programm ber-     */
/*              gebenen Parameter                    */
/*    /O, -O  : Bei bereinstimmung "OK" ausgeben    */
/*    /D, -D  : Bei Nichtbereinstimmung Lschen     */
/*              anbieten                             */
/*    /N, -N  : Lschen ohne Nachfrage               */
/*              (nur zusammen mit D)                 */
/* ------------------------------------------------- */
/*  Parameter                                        */
/*  Eingabe   : argc    : Anzahl der bergebenen     */
/*                        Parameter                  */
/*              *argv[] : Pointer auf die ber-      */
/*                        gebenen Parameter          */
/*  Ausgabe   : *source : Name der Schlsseldatei    */
/* ------------------------------------------------- */
void Parameter_auswerten(int argc, char *argv[],
                         char *source)
{
   int  zaehler;                  /* Schleifenzhler */

   strcpy(source, "");
   if ( (argc == 1) || (argv[1] == "/?") ||
        (argv[1] == "-?") )
     Anleitung_ausgeben();
   else {
     for (zaehler = 1; zaehler < argc; zaehler++) {
       strupr(argv[zaehler]);

       if ( (strcmp(argv[zaehler], "/O") == 0) ||
            (strcmp(argv[zaehler], "-O") == 0) )
         OK = TRUE;
       else if ( (strcmp(argv[zaehler], "/D") == 0) ||
                 (strcmp(argv[zaehler], "-D") == 0) )
         DELETE = TRUE;
       else if ( (strcmp(argv[zaehler], "/N") == 0) ||
                 (strcmp(argv[zaehler], "-N") == 0) )
         NOQUERY = TRUE;
       else strcpy(source, argv[zaehler]);
     }
   }

   if ((! strcmp(source, "")) ||
       (NOQUERY && ! DELETE) ||
       (source[0] == '/') || (source[0] == '-') )
     Anleitung_ausgeben();
} /* Parameter_auswerten */


/* ------------------------------------------------- */
/*  Modulname : Anleitung_ausgeben                   */
/*  Aufgabe   : bei fehlerhaftem Programmaufruf eine */
/*              Kurzanleitung ausgeben und Abbruch   */
/*              mit exit()                           */
/* ------------------------------------------------- */
/*  Parameter                                        */
/*  Eingabe   : Keine                                */
/*  Ausgabe   : mittels exit() wird der ERRORLEVEL   */
/*              von DOS gesetzt                      */
/* ------------------------------------------------- */
void  Anleitung_ausgeben(void)
{
  printf("\nCLEARHD - Dateien mit Schlsseldatei ");
  printf("vergleichen\n");
  printf("\nAufruf:   CLEARHD <Schlsseldateiname> ");
  printf("[/O] [/D [/N]]");
  printf("\n          /O - (OK)       fr korrekte ");
  printf("Dateien wird OK ausgegeben");
  printf("\n          /D - (Delete)   unkorrekte ");
  printf("Dateien nach Abfrage lschen");
  printf("\n          /N - (No query) unkorrekte ");
  printf("Dateien ohne Nachfrage lschen\n");

  exit(1);
         /* Programm abbrechen und ERRORLEVEL setzen */
} /* Anleitung_ausgeben */


/* ------------------------------------------------- */
/*  Modulname : vergleichen                          */
/*  Aufgabe   : Sequentielles Durchlaufen des ber-  */
/*              gebenen Directories.                 */
/*              Jede gefundene Datei wird in der     */
/*              Pointerliste gesucht.                */
/*              Wenn die Datei gefunden wird, wird   */
/*              sie aus der Pointerliste gestrichen. */                                         *
/*              Je nach gewhlten Optionen wird      */
/*      - bei bereinstimmungen OK ausgegeben        */
/*      - bei nicht in der Liste vorhandenen Dateien */
/*        das Lschen der entsprechenden Datei       */
/*        angeboten                                  */
/*        Bei falscher Gre, falscher Zeit und      */
/*        falschem Datum wird eine entsprechende     */
/*        Meldung  ausgegeben.                       */
/* ------------------------------------------------- */
/*  Parameter                                        */
/*   Eingabe : *rootptr: Zeiger auf den Listenanfang */
/*             *dir    : Name des zu durchsuchenden  */
/*                       Directory                   */
/*   Ausgabe : *rootptr: Zeiger auf den Listenanfang */
/* ------------------------------------------------- */
struct pointer *vergleichen(struct pointer *rootptr,
                            char *dir)
{
   char   loeschen;       /* Antwort auf Frage, ob   */
                          /* unkorrektes File        */
                          /* gelscht werden soll    */
   struct ffblk datei = {"",' ',0,0,0,""};
   char   helpstr[80],
          helpstr2[80],
          helpstr3[80],
          dateiname[13];
                       /* Dateiname des DIR-Eintrags */
   struct pointer *vorgaenger,
           /* Pointer zum Bearbeiten der Liste mit   */
                  *helpptr;  /*   den Dateieintrgen */

     /* Vergleich zwischen existierenden Dateien und */
     /* Eintrgen in Schlsseldatei                  */
   strcpy(helpstr,dir);
   if (helpstr[strlen(helpstr) - 1] != '\\')
         /* beim Hauptdirectory ist bereits ein '\\' */
     strcat(helpstr, "\\*.*");
         /*   vorhanden, bei Subdirectorys nicht     */
   else strcat(helpstr, "*.*");

   findfirst(helpstr,&datei,0x27);
     /* DOS: Ersten Fileeintrag im Directory suchen  */
   while (errno == 0)
     {   /* Noch ein Dateieintrag gefunden?          */
       mkstring(helpstr,datei);
         /* Aus Filedaten String machen              */
         /* TURBOC: Warning: Structure passed by     */
         /*                  value ist ok            */
       strcat(helpstr,"__");
            /* Notwendig fr exakten Stringvergleich */

       cutstr(dateiname, helpstr, 0, 12);
            /* Dateinamen aus helpstr schneiden      */

       /* In der Liste eine Datei mit gleichem       */
       /* Namen suchen                               */
       helpptr = rootptr;
       while ((helpptr != NULL) &&
          (strncmp(dateiname,helpptr->name, 12) != 0))
         {
           vorgaenger = helpptr;
          /* Filenamen sind ungleich -> weitersuchen */
           helpptr = helpptr->next;
         } /* while */

       if (helpptr != NULL)
                  /* Dateiname in der Liste gefunden */
       {
              /* Dateinamen isolieren und auf        */
              /* Gleichheit prfen                   */
         cutstr(helpstr2, helpptr->name, 0, 12);
         cutstr(helpstr3, helpstr, 0, 12);

         if (strcmp(helpstr2, helpstr3) == 0)
                       /* Eintrge absolut identisch */
         {
           if ((helpptr->name[39]=='C') ||
               (strcmp(helpptr->name, helpstr) == 0))
           {
             if (OK)
             {
          /* Filedaten gleich/drfen gendert werden */
               printf("\n%s",helpptr->name);
               printf("  OK");
             }
           }
           else {
             if (strcmp(helpptr->name, helpstr) != 0)
             {
                    /* LNGE, DATUM, ZEIT berprfen */
               printf("\n%s",helpptr->name);
                    /* Filestring ausgeben           */

               cutstr(helpstr2,helpstr,13,8);
                    /* Gre berprfen              */
               cutstr(helpstr3,helpptr->name,13,8);
               if (strcmp(helpstr2,helpstr3) != NULL)
                 printf("  WRONG SIZE");

               cutstr(helpstr2,helpstr,22,10);
                                 /* Datum berprfen */
               cutstr(helpstr3,helpptr->name,22,10);
               if (strcmp(helpstr2,helpstr3) != NULL)
                 printf("  WRONG DATE");

                 cutstr(helpstr2,helpstr,33,5);
                                /* Zeit berprfen   */
                 cutstr(helpstr3,helpptr->name,33,5);
                 if (strcmp(helpstr2,helpstr3) != NULL)
                   printf("  WRONG TIME");
                 }
               } /* else */

               if (helpptr != rootptr)
               {
                 /* Listenelement aus der Mitte der */
                 /* Liste                           */
                 vorgaenger->next = helpptr->next;
                 free(helpptr);
                 helpptr = vorgaenger->next;
               }
               else
               {
               /* Listenelement am Anfang der Liste */
                 rootptr = helpptr->next;
                 free(helpptr);
                 helpptr = rootptr;
                 vorgaenger = helpptr;
               } /* else */
             }
           }
           else
           {
             /* File nicht in Schlsseldatei, */
             /* Datei lschen ?               */
             printf("\n%s  *** Not required",helpstr);
             /* Filestring ausgeben */
             if ((DELETE) && (!NOQUERY))
             /* Nachfrage: Datei lschen? */
             {
               printf(" - KILL (Y/N)?");
               do
               {
                 loeschen = toupper(getche());
               }
               while ((loeschen != 'Y') &&
                      (loeschen != 'N'));
             }
             else printf(" ***");

             if ( (NOQUERY) ||
                  (DELETE && (loeschen =='Y')) )
             {
               strcpy(helpstr,dir);
                  /* beim Hauptdirectory ist bereits */
                  /* ein '\\' vorhanden,             */
                  /* bei Subdirectories nicht        */
               if (helpstr[strlen(helpstr) -1] != '\\')
                 strcat(helpstr,"\\");
               strcat(helpstr,datei.ff_name);
                           /* Dateiname nach helpstr */
               unlink(helpstr);
                           /* Datei lschen          */
               if (! errno)
                 printf("\n%40c  KILLED", ' ');
               else
                 printf("\n%40c  WRITE-PROTECTED",' ');
               errno = 0;
             };

       }  /* else */

       findnext(&datei);
        /* DOS: Nchsten Eintrag im Directory suchen */
     } /* while */

  return (rootptr);
} /* vergleichen */


/* ------------------------------------------------- */
/*  Modulname : cutstr                               */
/*  Aufgabe   : Ausschneiden eines Strings aus einem */
/*              Quellstring ab der Position position */
/*              und der maximalen Lnge length und   */
/*              bertragen in einen Zielstring.      */
/*              Wenn der Quellstring weniger Zeichen */
/*              als position+length besitzt, werden  */
/*              nur die Zeichen bis zum Quellstring- */
/*              ende zurckgegeben.                  */
/* ------------------------------------------------- */
/*  Parameter                                        */
/*   Eingabe : char *dest    Adresse des Zielstrings */
/*             char *source  Adresse des Quellstrings*/
/*             int  position ab welcher Position     */
/*                           ausgeschnitten werden   */
/*                           soll                    */
/*             int  length   wieviele Zeichen maximal*/
/*                           ausgeschnitten          */
/*                           werden sollen           */
/*  Ausgabe  : Anzahl der ausgeschnittenen Zeichen   */
/* ------------------------------------------------- */
int cutstr(char *dest, char *source,
           int position, int length)
{
  int  i = 0;        /* Schleifenvariable */

  while ((i<length) &&
         (source[position+i] != '\0')) {
    dest[i] = source[position+i];
    i++;
  }
  dest[i] = '\0';
  return(i);
} /* cutstr */

/* ------------------------------------------------- */
/*              Ende von CLEARHD.C                   */

